@@ -17,6 +17,8 @@ import os.path
 import mimetypes
 import urlparse
 import urllib
+import warnings
+from unicodedata import normalize
 
 from werkzeug.exceptions import HTTPException
 from flask import Flask, Module, url_for, request, send_from_directory
@@ -34,9 +36,13 @@ except ImportError:
 __all__ = ['Freezer']
 
 
+class MissingURLGeneratorWarning(Warning):
+    pass
+
+
 class Freezer(object):
     """
-    :param app: your application
+    :param app: your application or None if you use :meth:`init_app`
     :type app: Flask instance
     :param with_static_files: Whether to automatically generate URLs
                               for static files.
@@ -45,16 +51,26 @@ class Freezer(object):
                                    for URL rules that take no arguments.
     :type with_no_argument_rules boolean:
     """
-    def __init__(self, app, with_static_files=True,
+    def __init__(self, app=None, with_static_files=True,
                  with_no_argument_rules=True):
-        app.config.setdefault('FREEZER_DESTINATION', 'build')
-        app.config.setdefault('FREEZER_BASE_URL', 'http://localhost/')
-        self.app = app
         self.url_generators = []
         if with_static_files:
             self.register_generator(self.static_files_urls)
         if with_no_argument_rules:
             self.register_generator(self.no_argument_rules_urls)
+        self.init_app(app)
+
+    def init_app(self, app):
+        """
+        Allow to register an app after the Freezer initialization.
+
+        :param app: your Flask application
+        """
+        self.app = app
+        if app:
+            app.config.setdefault('FREEZER_DESTINATION', 'build')
+            app.config.setdefault('FREEZER_BASE_URL', 'http://localhost/')
+            app.config.setdefault('FREEZER_REMOVE_EXTRA_FILES', True)
 
     def register_generator(self, function):
         """Register a function as an URL generator.
@@ -74,14 +90,18 @@ class Freezer(object):
         """The build destination."""
         # unicode() will raise if the path is not ASCII or already unicode.
         return os.path.join(
-            self.app.root_path,
-            self.app.config['FREEZER_DESTINATION']
+            unicode(self.app.root_path),
+            unicode(self.app.config['FREEZER_DESTINATION'])
         )
 
     def freeze(self):
         """Clean the destination and build all URLs from generators."""
+        remove_extra = self.app.config['FREEZER_REMOVE_EXTRA_FILES']
+        if not os.path.isdir(self.root):
+            os.makedirs(self.root)
         previous_files = set(
-            os.path.join(self.root, *name.split('/'))
+            # See https://github.com/SimonSapin/Frozen-Flask/issues/5
+            normalize('NFC', os.path.join(self.root, *name.split('/')))
             for name in walk_directory(self.root)
         )
         seen_urls = set()
@@ -92,14 +112,15 @@ class Freezer(object):
                 continue
             seen_urls.add(url)
             new_filename = self._build_one(url)
-            built_files.add(new_filename)
-        # Remove files from the previous build that are not here anymore.
-        for extra_file in previous_files - built_files:
-            os.remove(extra_file)
-            parent = os.path.dirname(extra_file)
-            if not os.listdir(parent):
-                # The directory is now empty, remove it.
-                os.removedirs(parent)
+            built_files.add(normalize('NFC', new_filename))
+        if remove_extra:
+            # Remove files from the previous build that are not here anymore.
+            for extra_file in previous_files - built_files:
+                os.remove(extra_file)
+                parent = os.path.dirname(extra_file)
+                if not os.listdir(parent):
+                    # The directory is now empty, remove it.
+                    os.removedirs(parent)
         return seen_urls
 
     def all_urls(self):
@@ -109,6 +130,8 @@ class Freezer(object):
         """
         base_url = self.app.config['FREEZER_BASE_URL']
         script_name = urlparse.urlsplit(base_url).path.rstrip('/')
+        seen_endpoints = set()
+        url_encoding = self.app.url_map.charset
         # A request context is required to use url_for
         with self.app.test_request_context(base_url=script_name):
             for generator in self.url_generators:
@@ -124,6 +147,7 @@ class Freezer(object):
                         else:
                             # Assume a tuple.
                             endpoint, values = generated
+                        seen_endpoints.add(endpoint)
                         url = url_for(endpoint, **values)
                         assert url.startswith(script_name), (
                             'url_for returned an URL %r not starting with '
@@ -131,12 +155,32 @@ class Freezer(object):
                             % (url, script_name)
                         )
                         url = url[len(script_name):]
-                    # Flask.url_for "quotes" URLs, eg. a space becomes %20
+                    # flask.url_for "quotes" URLs, eg. a space becomes %20
                     url = urllib.unquote(url)
-                    assert not url.startswith(('http:', 'https:')), \
-                        'External URLs not supported: ' + url
+                    parsed_url = urlparse.urlsplit(url)
+                    if parsed_url.scheme or parsed_url.netloc:
+                        raise ValueError('External URLs not supported: ' + url)
+
+                    if not isinstance(url, unicode):
+                        url = url.decode(url_encoding)
                     yield url
 
+        all_endpoints = set(
+            rule.endpoint for rule in self.app.url_map.iter_rules())
+        not_generated_endpoints = all_endpoints - seen_endpoints
+
+        if self.static_files_urls in self.url_generators:
+            # Special case: do not warn when there is no static file
+            not_generated_endpoints -= set(self._static_rules_endpoints())
+
+        if not_generated_endpoints:
+            warnings.warn(
+                'Nothing frozen for endpoints %s. Did you forget an URL '
+                'generator?' % ', '.join(
+                    unicode(e) for e in not_generated_endpoints),
+                MissingURLGeneratorWarning,
+                stacklevel=3)
+
     def _build_one(self, url):
         """Get the given ``url`` from the app and write the matching file.
         """
@@ -145,8 +189,9 @@ class Freezer(object):
         response = client.get(url, follow_redirects=True, base_url=base_url)
         # The client follows redirects by itself
         # Any other status code is probably an error
-        assert response.status_code == 200, 'Unexpected status %r on URL %s' \
-            % (response.status, url)
+        if not(response.status_code == 200):
+            raise ValueError('Unexpected status %r on URL %s' \
+                % (response.status, url))
 
         destination_path = url + 'index.html' if url.endswith('/') else url
 
@@ -158,10 +203,10 @@ class Freezer(object):
         if not guessed_type:
             # Used by most server when they can not determine the type
             guessed_type = 'application/octet-stream'
-        assert guessed_type == response.mimetype, (
-            'Filename extension of %r (type %s) does not match Content-Type:'
-            ' %s' % (basename, guessed_type, response.content_type)
-        )
+        if not guessed_type == response.mimetype:
+            raise ValueError(
+                'Filename extension of %r (type %s) does not match Content-'
+                'Type: %s' % (basename, guessed_type, response.content_type))
 
         # Remove the initial slash that should always be there
         assert destination_path[0] == '/'
@@ -219,8 +264,9 @@ class Freezer(object):
         app.dispatch_request = dispatch_request
         return app
 
-    def static_files_urls(self):
-        """URL generator for static files for app and all registered modules.
+    def _static_rules_endpoints(self):
+        """
+        Yield the 'static' URL rules for the app and all modules.
         """
         send_static_file = unwrap_method(Flask.send_static_file)
         # Assumption about a Flask internal detail:
@@ -228,18 +274,24 @@ class Freezer(object):
         # This will break loudly if the assumption isn't valid anymore in
         # a future version of Flask
         assert unwrap_method(Module.send_static_file) is send_static_file
-        
+
         for rule in self.app.url_map.iter_rules():
             view = self.app.view_functions[rule.endpoint]
-            if unwrap_method(view) is not send_static_file:
-                continue
-            # Found an URL rule for the static files view
-            root = os.path.join(method_self(view).root_path, 'static')
+            if unwrap_method(view) is send_static_file:
+                yield rule.endpoint
+
+    def static_files_urls(self):
+        """URL generator for static files for app and all registered modules.
+        """
+        for endpoint in self._static_rules_endpoints():
+            view = self.app.view_functions[endpoint]
+            app_or_module = method_self(view)
+            root = os.path.join(app_or_module.root_path, 'static')
             if not os.path.isdir(root):
                 # No 'static' directory for this app/module.
                 continue
             for filename in walk_directory(root):
-                yield rule.endpoint, {'filename': filename}
+                yield endpoint, {'filename': filename}
 
     def no_argument_rules_urls(self):
         """URL generator for URL rules that take no arguments."""
@@ -252,7 +304,7 @@ def walk_directory(root):
     """
     Recursively walk the `root` directory and yield slash-separated paths
     relative to the root.
-    
+
     Used to implement the URL genertor for static files.
     """
     for name in os.listdir(root):
@@ -286,4 +338,3 @@ def method_self(method):
     except AttributeError:
         # Python 3
         return method.__self__
-
